@@ -3,7 +3,7 @@ package ObjectDB::DBHPool;
 use strict;
 use warnings;
 
-our $VERSION = '3.20';
+our $VERSION = '3.21';
 
 require Carp;
 use ObjectDB::DBHPool::Connection;
@@ -35,15 +35,14 @@ sub dbh {
     $pid_tid .= '_' . threads->tid
       if exists $INC{'threads.pm'} && $INC{'threads.pm'};
 
-    my $connection = $self->{connections}->{$pid_tid} ||=
-      ObjectDB::DBHPool::Connection->new(
+    my $connection = $self->{connections}->{$pid_tid} ||= ObjectDB::DBHPool::Connection->new(
         check_timeout => $self->{check_timeout},
         dsn           => $self->{dsn},
         username      => $self->{username},
         password      => $self->{password},
         attrs         => $self->{attrs},
         do            => $self->{do},
-      );
+    );
 
     return $connection->dbh;
 }
