@@ -5,9 +5,10 @@ use warnings;
 
 use base 'ObjectDB::Related';
 
-our $VERSION = '3.20';
+our $VERSION = '3.21';
 
-use ObjectDB::Util qw(merge);
+use Storable qw(dclone);
+use ObjectDB::Util qw(to_array);
 
 sub create_related {
     my $self = shift;
@@ -29,10 +30,10 @@ sub create_related {
         my $map_to   = $meta->map_to;
 
         my ($from_foreign_pk, $from_pk) =
-          %{$meta->map_class->meta->get_relationship($map_from)->map};
+          %{ $meta->map_class->meta->get_relationship($map_from)->map };
 
         my ($to_foreign_pk, $to_pk) =
-          %{$meta->map_class->meta->get_relationship($map_to)->map};
+          %{ $meta->map_class->meta->get_relationship($map_to)->map };
 
         my $map_object = $meta->map_class->new(
             $from_foreign_pk => $row->get_column($from_pk),
@@ -71,8 +72,9 @@ sub _related_table     { shift->meta->class->table }
 sub _related_map_table { shift->meta->map_class->table }
 
 sub _build_params {
-    my $self = shift;
-    my ($row) = shift;
+    my $self     = shift;
+    my ($row)    = shift;
+    my (%params) = @_;
 
     my $meta = $self->meta;
 
@@ -80,14 +82,15 @@ sub _build_params {
     my $map_to   = $meta->map_to;
 
     my ($map_table_to, $map_table_from) =
-      %{$meta->map_class->meta->get_relationship($map_from)->map};
+      %{ $meta->map_class->meta->get_relationship($map_from)->map };
 
     my $table     = $meta->class->meta->table;
     my $map_table = $meta->map_class->meta->table;
 
-    my $params = merge { @_ },
-      {where => ["$map_table.$map_table_to" => $row->column($map_table_from)]};
-    return %$params;
+    my $merged = dclone(\%params);
+    $merged->{where} = [ "$map_table.$map_table_to" => $row->column($map_table_from), to_array $merged->{where} ];
+
+    return %$merged;
 }
 
 1;
