@@ -5,10 +5,11 @@ use warnings;
 
 use base 'ObjectDB::Related::ManyToOne';
 
-our $VERSION = '3.20';
+our $VERSION = '3.21';
 
 use Scalar::Util ();
-use ObjectDB::Util qw(merge);
+use Storable qw(dclone);
+use ObjectDB::Util qw(to_array);
 
 sub create_related {
     my $self = shift;
@@ -19,7 +20,7 @@ sub create_related {
     }
 
     my $meta = $self->meta;
-    my ($from, $to) = %{$meta->map};
+    my ($from, $to) = %{ $meta->map };
 
     my @where = ($to => $row->column($from));
 
@@ -52,14 +53,17 @@ sub delete_related {
 sub _related_table { shift->meta->class->table }
 
 sub _build_params {
-    my $self = shift;
-    my ($row) = shift;
+    my $self     = shift;
+    my ($row)    = shift;
+    my (%params) = @_;
 
     my $meta = $self->meta;
-    my ($from, $to) = %{$meta->map};
+    my ($from, $to) = %{ $meta->map };
 
-    my $params = merge { @_ }, {where => [$to => $row->column($from)]};
-    return %$params;
+    my $merged = dclone(\%params);
+    $merged->{where} = [ $to => $row->column($from), to_array $merged->{where} ];
+
+    return %$merged;
 }
 
 1;
