@@ -6,55 +6,62 @@ use warnings;
 use TestDBH;
 
 my %TABLES = (
-    'person' => <<'',
+    'person' => <<'EOS',
     CREATE TABLE `person` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `name` varchar(40) DEFAULT '',
      `profession` varchar(40),
      `age` INTEGER
     );
+EOS
 
-    'author' => <<'',
+    'author' => <<'EOS',
     CREATE TABLE `author` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `name` varchar(40)
     );
+EOS
 
-    'book' => <<'',
+    'book' => <<'EOS',
     CREATE TABLE `book` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `author_id` INTEGER NOT NULL DEFAULT 0,
      `title` varchar(40)
     );
+EOS
 
-    'book_description' => <<'',
+    'book_description' => <<'EOS',
     CREATE TABLE `book_description` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `book_id` INTEGER NOT NULL DEFAULT 0,
      `description` varchar(40)
     );
+EOS
 
-    'tag' => <<'',
+    'tag' => <<'EOS',
     CREATE TABLE `tag` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `name` varchar(40)
     );
+EOS
 
-    'book_tag_map' => <<'',
+    'book_tag_map' => <<'EOS',
     CREATE TABLE `book_tag_map` (
      `book_id` INTEGER,
      `tag_id` INTEGER,
      PRIMARY KEY(`book_id`, `tag_id`)
     );
+EOS
 
-    'thread' => <<'',
+    'thread' => <<'EOS',
     CREATE TABLE `thread` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `author_id` INTEGER,
      `title` varchar(40)
     );
+EOS
 
-    'reply' => <<'',
+    'reply' => <<'EOS',
     CREATE TABLE `reply` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `author_id` INTEGER,
@@ -62,14 +69,16 @@ my %TABLES = (
      `parent_id` INTEGER,
      `content` varchar(40)
     );
+EOS
 
-    'notification' => <<'',
+    'notification' => <<'EOS',
     CREATE TABLE `notification` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `reply_id` INTEGER
     );
+EOS
 
-    'auto' => <<'',
+    'auto' => <<'EOS',
     CREATE TABLE `auto` (
      `id` INTEGER PRIMARY KEY AUTOINCREMENT,
      `varchar_no_default` varchar(40),
@@ -82,7 +91,7 @@ my %TABLES = (
      `bool_default_false` BOOLEAN DEFAULT 0,
      `bool_default_true` BOOLEAN DEFAULT 1
     );
-
+EOS
 );
 
 sub prepare_table {
@@ -101,7 +110,8 @@ sub prepare_table {
 
     if ($driver =~ /mysql/i) {
         $sql =~ s{AUTOINCREMENT}{AUTO_INCREMENT}g;
-    } elsif ($driver =~ /Pg/i) {
+    }
+    elsif ($driver =~ /Pg/i) {
         $sql =~ s{`}{"}g;
         $sql =~ s{"id" INTEGER PRIMARY KEY AUTOINCREMENT}{"id" SERIAL PRIMARY KEY}g;
         $sql =~ s{BOOLEAN DEFAULT 0}{BOOLEAN DEFAULT 'f'}g;
